@@ -25,7 +25,7 @@ rgl.Sweave <- function(name, width, height, options, ...) {
     wrnew <- par3d("windowRect")
     if (wr[3] - wr[1] != wrnew[3] - wrnew[1] || 
         wr[4] - wr[2] != wrnew[4] - wrnew[2])
-      stop("rgl window creation error.  Try reducing resolution, width or height.")
+      stop("rgl window creation error; try reducing resolution, width or height")
     dev <- rgl.cur()
   } 
   
@@ -46,7 +46,7 @@ rgl.Sweave <- function(name, width, height, options, ...) {
         png = rgl.snapshot(filename=paste(name, "png", sep=".")),
         pdf = rgl.postscript(filename=paste(name, "pdf", sep="."), fmt="pdf"),
         eps = rgl.postscript(filename=paste(name, "eps", sep="."), fmt="eps"),
-        stop("Unrecognized rgl outputtype: ", type)
+        stop(gettextf("Unrecognized rgl outputtype: '%s'", type), domain = NA)
       )
       snapshotDone <<- TRUE
     }
@@ -87,20 +87,30 @@ hook_webgl <- local({
   reuse <- TRUE
   function (before, options, envir) 
   {
-    if (before || rgl.cur() == 0)
+    if (before) {
+      newwindow <- options$rgl.newwindow
+      if (!is.null(newwindow) && newwindow) 
+      	open3d()
+      if (!is.null(options$rgl.keepopen))
+      	warning("rgl.keepopen has been replaced by rgl.newwindow")
       return()
+    } else if (rgl.cur() == 0)
+      return()    
+    	
     if (requireNamespace("knitr")) { # Should we stop if there is no knitr?
                                      # We only use it in this test.
       out_type <- knitr::opts_knit$get("out.format")
       if (!length(intersect(out_type, c("markdown", "html"))))       
-        stop("hook_webgl is for HTML only.  Use hook_rgl instead.")
+        stop("'hook_webgl' is for HTML only.  Use 'hook_rgl' instead.")
     }  
     name <- tempfile("webgl", tmpdir = ".", fileext = ".html")
     on.exit(unlink(name))
     retina <- options$fig.retina
     if (!is.numeric(retina)) retina <- 1 # It might be FALSE or maybe NULL
     dpi <- options$dpi / retina  # should not consider Retina displays (knitr #901)
-    par3d(windowRect = 100 + dpi * c(0, 0, options$fig.width, 
+    margin <- options$rgl.margin
+    if (is.null(margin)) margin <- 100
+    par3d(windowRect = margin + dpi * c(0, 0, options$fig.width, 
                                            options$fig.height))
     Sys.sleep(.05) # need time to respond to window size change
     
@@ -113,8 +123,6 @@ hook_webgl <- local({
                     prefix = prefix, 
                     commonParts = commonParts,
                     reuse = reuse)
-    if (!isTRUE(options$rgl.keepopen) && rgl.cur())
-      rgl.close()
     commonParts <<- FALSE
     reuse <<- attr(res, "reuse")
     res <- readLines(name)
@@ -123,14 +131,24 @@ hook_webgl <- local({
   }
 })
 
-hook_rgl = function(before, options, envir) {
-  # after a chunk has been evaluated
-  if (before || rgl.cur() == 0) return()  # no active device
+hook_rgl <- function(before, options, envir) {
+  if (before) {
+    newwindow <- options$rgl.newwindow
+    if (!is.null(newwindow) && newwindow) 
+      open3d()
+    if (!is.null(options$rgl.keepopen))       
+      warning("rgl.keepopen has been replaced by rgl.newwindow")
+    return()
+  } else if (rgl.cur() == 0)
+    return()
+	
   if (!requireNamespace("knitr")) 
-    stop("hook_rgl requires the 'knitr' package.")
+    stop("'hook_rgl' requires the 'knitr' package.")
 
   name <- knitr::fig_path('', options)
-  par3d(windowRect = 100 + options$dpi * c(0, 0, options$fig.width, options$fig.height))
+  margin <- options$rgl.margin
+  if (is.null(margin)) margin <- 100
+  par3d(windowRect = margin + options$dpi * c(0, 0, options$fig.width, options$fig.height))
   Sys.sleep(.05) # need time to respond to window size change
 
   dir <- knitr::opts_knit$get('base_dir')
@@ -146,13 +164,23 @@ hook_rgl = function(before, options, envir) {
   knitr::hook_plot_custom(before, options, envir)
 }
 
-save_rgl = function(name, devices) {
+save_rgl <- function(name, devices) {
   if (!file_test('-d', dirname(name))) dir.create(dirname(name), recursive = TRUE)
   # support 3 formats: eps, pdf and png (default)
   for (dev in devices) switch(
     dev,
+    eps =,
     postscript = rgl.postscript(paste0(name, '.eps'), fmt = 'eps'),
     pdf = rgl.postscript(paste0(name, '.pdf'), fmt = 'pdf'),
     rgl.snapshot(paste0(name, '.png'), fmt = 'png')
   )
 }
+
+setupKnitr <- function() {
+  if (requireNamespace("knitr")) {
+    knitr::knit_hooks$set(webgl = hook_webgl)
+    knitr::knit_hooks$set(rgl = hook_rgl)
+    environment(hook_webgl)$commonParts <- TRUE
+    environment(hook_webgl)$reuse <- TRUE
+  }
+}
