@@ -1,7 +1,7 @@
 options(rgl.useNULL=TRUE)
 library(rgl)
 
-knitr::knit_hooks$set(rgl = hook_webgl)
+setupKnitr()
 
 if (requireNamespace("rmarkdown") && !rmarkdown::pandoc_available("1.13.1"))
   warning("These vignettes assume pandoc version 1.13.1; older versions may give poor formatting.")
